@@ -45,9 +45,11 @@ TclpDlopen(
 				 * file. */
     int flags)
 {
-    Tcl_SetObjResult(interp, Tcl_NewStringObj(
-	    "dynamic loading is not currently available on this system",
-	    -1));
+    if (interp) {
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
+		"dynamic loading is not currently available on this system",
+		-1));
+    }
     return TCL_ERROR;
 }
 
@@ -104,11 +106,15 @@ TclpLoadMemory(
     int size,			/* Dummy: unused by this implementation */
     int codeSize,		/* Dummy: unused by this implementation */
     Tcl_LoadHandle *loadHandle,	/* Dummy: unused by this implementation */
-    Tcl_FSUnloadFileProc **unloadProcPtr)
+    Tcl_FSUnloadFileProc **unloadProcPtr,
+				/* Dummy: unused by this implementation */
+    int flags)
 				/* Dummy: unused by this implementation */
 {
-    Tcl_SetObjResult(interp, Tcl_NewStringObj("dynamic loading from memory "
-	    "is not available on this system", -1));
+    if (interp) {
+	Tcl_SetObjResult(interp, Tcl_NewStringObj("dynamic loading from memory "
+		"is not available on this system", -1));
+    }
     return TCL_ERROR;
 }
 
