@@ -136,7 +136,7 @@ TclpGetClicks(void)
 	Tcl_Time time;
 
 	tclGetTimeProcPtr(&time, tclTimeClientData);
-	now = time.sec*1000000 + time.usec;
+	now = ((unsigned long)(time.sec)*1000000UL) + (unsigned long)(time.usec);
     } else {
 	/*
 	 * A semi-NativeGetTime, specialized to clicks.
@@ -149,7 +149,7 @@ TclpGetClicks(void)
     Tcl_Time time;
 
     tclGetTimeProcPtr(&time, tclTimeClientData);
-    now = time.sec*1000000 + time.usec;
+    now = ((unsigned long)(time.sec)*1000000UL) + (unsigned long)(time.usec);
 #endif
 
     return now;
